@Echo off
REM Licensed Materials - Property of IBM
REM IBM Cognos Products: CA
REM (C) Copyright IBM Corp. 2009, 2017
REM US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

Title IIS Configuration Script v2

:: Check to see if running as Administrator
net session >nul 2>&1
if %errorLevel% == 0 (
    GOTO set_variables
) else (
    echo Please use 'Run an Administrator' to run this tool.
	GOTO eof
)

:set_variables
setlocal enabledelayedexpansion

:: Variables
set x=2
set appcmd_path=%SystemRoot%\system32\inetsrv\appcmd.exe
set kit_ver=null
set prod_ver=null
set CA_gate=false

:: set Default web site name
:: This is the name of the web site that the CA alias is created in (ex. Default Web Site)
:: web_protocol can be set to http ot https and will only be use if a web server farm is set.
::
set web_site=Default Web Site

:: set Application Pool Name
:: Name of the Application Pool that will be used.
::
set app_pool=ICAPool

:: set Cognos Analytics alias
:: This is the web alias you will use for Cognos Analytics.
::
set alias=ibmcognos

:: Location of the Cognos Analytics gateway folder
:: Type in the path to the Cognos Analytics gateway install (ex. C:\Program Files\ibm\cognos\analytics)
::
set ca_path=C:\Program Files\ibm\cognos\analytics

:: CA dispatcher servers (Only Application Servers - Do not list Content Manager only server)
::
:: Set the disp.protocol to http or https
:: The disp.protocol will set the reverse proxy url to the proper protocol.
:: All dispatchers must have the same protocol set
::
set disp.protocol=http

:: When adding more than one server to the listing, increase disp[x] by 1.  Always start at 1.
:: If 2 or more, the the port number will set in the correct location for the server farm.

:: For Example
::     set disp[1].name=server1_name
::     set disp[1].port=9300
::     set disp[2].name=server2_name
::     set disp[2].port=9400
::     set disp[3].name=server3_name
::     set disp[3].port=9500
::     etc....
::
:: If more than one dispatcher is defined, a Server Farm will be created
::
set disp[1].name=localhost
set disp[1].port=9300

:: Enable SSO (True/False)
:: 
set enable_SSO=False

:: set Web Farm name
:: This name will only be used if there are more than 1 dispatcher listed.
::
set web_farm_name=ca_servers

:: Path location validation
:: This section will search to see if the cmplst.txt file exists in the ca_path location
:: If file exists, it will check to see prod version and if the gateway tier is installed.

if exist "%ca_path%\cmplst.txt" (goto :CA_found) else (goto :CA_notfound)

:CA_found
for /F "delims=" %%a in ('findstr "C8GATE_name=Gateway C8GATE_version=" "%ca_path%\cmplst.txt"') do set "CA_gate=true"
for /F "delims=" %%b in ('findstr /b "kit_version Product_version" "%ca_path%\cmplst.txt"') do set "kit_ver=%%b"
set "kit_result=%kit_ver:n=" & set "kit_result=%"
if %CA_gate% equ true (set prod_ver=PATH VALID: FOUND CA GATEWAY %kit_result:~1%) else (set prod_ver=ERROR - NO GATEWAY TIER INSTALLED AT THIS LOCATION)
goto :menu

:CA_notfound
set prod_ver=PATH ERROR - Re-check the CA Gateway Location path.

:: *******************************************************************
:menu
cls
echo.
echo IIS Configuration Script for IBM Cognos Analytics
echo *****************************************************************
echo.
echo    Web Site Name                     - %web_site%
echo    Alias Name                        - %alias%
echo.
echo    Application Pool Name             - %app_pool%
echo.
echo    CA Gateway Location               - %ca_path%
echo                                        %prod_ver%
echo.
echo    CA Dispatcher(s) Protocol         - {!disp.protocol!}
echo    CA Dispatcher(s) and Port#(s)     - !disp[1].name!:!disp[1].port!

:DescLoop
:: Used when dispatcher are greater than 1
if defined disp[%x%].name ( 
   @echo                                      - !disp[%x%].name!:!disp[%x%].port!
   set /a "x+=1"
   GOTO :DescLoop 
   )

echo.
echo    SSO Enabled                       - %enable_SSO%
echo.

if %x% gtr 2 (
   @echo    Server Farm Name                  - %web_farm_name%
   @echo.
   )

echo *****************************************************************
echo.
echo If values above are correct - press 'y' to continue.
echo If values above are incorrect - press 'n' to exit.
echo.

CHOICE /N /M "What is your choice (y|n)"

IF ERRORLEVEL ==2 GOTO err_exit
IF ERRORLEVEL ==1 GOTO config_iis

::*******************************************************************

:err_exit
echo.
echo IIS configuration script is now exiting
goto eof

:config_iis
echo.
echo --- Enable ARR Proxy
%appcmd_path% set config  -section:system.webServer/proxy /enabled:"True"  /commit:apphost 1>nul

echo --- Creating Application Pool
%appcmd_path% add apppool /name:"%app_pool%" /managedRuntimeVersion:v4.0 1>nul
%appcmd_path% set config  -section:system.applicationHost/applicationPools /[name='%app_pool%'].enable32BitAppOnWin64:"False"  /commit:apphost 1>nul

echo --- Creating Alias
%appcmd_path% add app /site.name:"%web_site%" /path:/%alias% /physicalpath:"%ca_path%\webcontent" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/httpErrors /existingResponse:"PassThrough" 1>nul

echo --- Assigning alias to application pool
%appcmd_path% set app "%web_site%/%alias%" /applicationpool:%app_pool% 1>nul

:: Create the server farm
if %x% gtr 2 (goto create_wf) 1>nul
goto create_wp

:: #######
:: If there is more than one dispatcher defined, this routine will execute to create the server farm
:create_wf
echo --- Creating Server Farm
%appcmd_path% set config  -section:webFarms /+"[name='%web_farm_name%']" /commit:apphost 1>nul

set "x=1"
:add_disp 
if defined disp[%x%].name ( 
   %appcmd_path% set config  -section:webFarms /+"[name='%web_farm_name%'].[address='!disp[%x%].name!']" /commit:apphost 1>nul
   %appcmd_path% set config  -section:webFarms /[name='%web_farm_name%'].[address='!disp[%x%].name!'].applicationRequestRouting.!disp.protocol!Port:"!disp[%x%].port!"  /commit:apphost 1>nul
   set /a "x+=1"
   GOTO :add_disp 
)

%appcmd_path% set config  -section:webFarms /[name='%web_farm_name%'].applicationRequestRouting.affinity.useCookie:"True"  /commit:apphost 1>nul
%appcmd_path% set config  -section:webFarms /[name='%web_farm_name%'].applicationRequestRouting.healthCheck.url:"!disp.protocol!://%web_farm_name%/bi/v1/ping"  /commit:apphost 1>nul
%appcmd_path% set config  -section:webFarms /[name='%web_farm_name%'].applicationRequestRouting.protocol.cache.queryStringHandling:"Accept"  /commit:apphost 1>nul
%appcmd_path% set config  -section:webFarms /[name='%web_farm_name%'].applicationRequestRouting.protocol.timeout:"00:02:00"  /commit:apphost 1>nul
:: #######

:create_wp
echo --- Adding Mime Types
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /-"[fileExtension='.woff2']" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /-"[fileExtension='.woff']" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /-"[fileExtension='.svg']" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /-"[fileExtension='.json']" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /-"[fileExtension='.template']" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /-"[fileExtension='.txt']" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /+"[fileExtension='.svg',mimeType='image/svg%%u002bxml']" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /+"[fileExtension='.woff',mimeType='application/x-font-woff']" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /+"[fileExtension='.json',mimeType='application/json']" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /+"[fileExtension='.woff2',mimeType='font/woff2']" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /+"[fileExtension='.template',mimeType='text/html']" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /+"[fileExtension='.txt',mimeType='text/plain']" 1>nul

echo --- Adjusting request size limits
%appcmd_path% set config "%web_site%/%alias%" -section:system.web/httpRuntime /maxQueryStringLength:"8192" 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" /section:requestFiltering /requestLimits.maxQueryString:8192 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" /section:requestFiltering /requestLimits.maxUrl:8192 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" /section:requestFiltering /requestLimits.maxAllowedContentLength:500000000 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/security/requestFiltering /+"requestLimits.headerLimits.[header='Referer',sizeLimit='8192']" 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/security/requestFiltering /+"requestLimits.headerLimits.[header='Cookie',sizeLimit='4096']" 1>nul

echo --- HTTP responce headers: Expire Web content
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /clientCache.cacheControlMode:"UseMaxAge" 1>nul
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/staticContent /clientCache.cacheControlMaxAge:"1.00:00:00" 1>nul

echo --- Creating SSO application and assigning application pool
%appcmd_path% add app /site.name:"%web_site%" /path:/%alias%/sso /physicalpath:"%ca_path%\cgi-bin" 1>nul
%appcmd_path% set app "%web_site%/%alias%/sso" /applicationpool:%app_pool% 1>nul
%appcmd_path% set config "%web_site%/%alias%/sso" -section:system.webServer/security/authentication/anonymousAuthentication /enabled:"False"  /commit:apphost 1>nul
%appcmd_path% set config "%web_site%/%alias%/sso" -section:system.webServer/security/authentication/windowsAuthentication /enabled:"True"  /commit:apphost 1>nul

echo --- Creating handler map
%appcmd_path% set config "%web_site%/%alias%/sso" -section:system.webServer/handlers /+"[name='Cognos SSO',path='cisapi',verb='*',modules='IsapiModule',scriptProcessor='%ca_path%\cgi-bin\cognosisapi.dll',preCondition='bitness64']" /commit:apphost 1>nul
%appcmd_path% set config -section:system.webServer/security/isapiCgiRestriction /+"[path='%ca_path%\cgi-bin\cognosisapi.dll',allowed='True',groupId='Cognos',description='IBM Cognos Analytics Isapi']" /commit:apphost 1>nul
%appcmd_path% set config "%web_site%/%alias%/sso" -section:system.webServer/handlers /"[name='Cognos SSO'].resourceType:"Unspecified"" /"[name='Cognos SSO'].requireAccess:"Script"" 1>nul

echo --- Removing webdav module from SSO application
%appcmd_path% set config "%web_site%/%alias%/sso" -section:system.webServer/modules /-"[name='WebDAVModule']" /commit:apphost 1>nul

echo --- Allow pass through for custom 441 errors
%appcmd_path% set config "%web_site%/%alias%" -section:system.webServer/httpErrors /existingResponse:"PassThrough"  1>nul

echo.
echo --- CREATING RULES

echo ........Server Variables
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/allowedServerVariables /+"[name='HTTP_X_BI_PATH']" /commit:apphost 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/allowedServerVariables /+"[name='HTTP_X_FORWARDED_HOST']" /commit:apphost 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/allowedServerVariables /+"[name='HTTP_X_WEBCONTENTROOT']" /commit:apphost 1>nul

echo ........Headers
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /+"[@start,name='Headers',stopProcessing='False']"  1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /[name='Headers'].match.url:"(.*)" 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Headers'].action.type:"None"" /commit  1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /-"[name='Headers'].serverVariables.[name='HTTP_X_BI_PATH']" 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /+"[name='Headers'].serverVariables.[name='HTTP_X_BI_PATH',value='/%alias%/bi/v1']" 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /+"[name='Headers'].serverVariables.[name='HTTP_X_FORWARDED_HOST',value='{HTTP_HOST}']" 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /+"[name='Headers'].serverVariables.[name='HTTP_X_WEBCONTENTROOT',value='/%alias%']" 1>nul

echo ........SSO Login - %enable_SSO%
::  Will be used only if SSO has been enabled.
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /+"[@1,name='SSO Login',stopProcessing='True']"  1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='SSO Login',stopProcessing='True'].match.url:"v1/login$""  1>nul 
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='SSO Login',stopProcessing='True'].action.type:"Rewrite"" "/[name='SSO Login',stopProcessing='True'].action.url:"/%alias%/sso/cisapi/bi/v1/login"" 1>nul

echo ........Legacy SSO - %enable_SSO%
::  Will be used only if SSO has been enabled.
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /+"[@2,name='Legacy SSO',stopProcessing='True']"  1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Legacy SSO',stopProcessing='True'].match.url:(v1/disp(/.*)?)"  1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Legacy SSO',stopProcessing='True'].action.type:"Rewrite"" "/[name='Legacy SSO',stopProcessing='True'].action.url:"/%alias%/sso/cisapi/bi/{R:1}"" 1>nul

echo ........Reverse Proxy
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /+"[@3,name='Reverse Proxy']" /commit 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Reverse Proxy'].match.url:(^$)|(^v1(/.*)?)|(^[^/]+\.jsp)" /commit 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Reverse Proxy'].action.type:"Rewrite"" /commit 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Reverse Proxy'].stopProcessing:"true"" /commit 1>nul

:: #######
:: Logic to set dispatcher or web farm name in the reverse Proxy rule.
if %x% gtr 2 goto enable_wf
	echo ............Dispatcher set
	%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Reverse Proxy'].action.url:"!disp.protocol!://!disp[1].name!:!disp[1].port!/bi/{R:0}"" /commit 1>nul
goto skip_wf

:enable_wf
echo ............Server Farm set
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Reverse Proxy'].action.url:"!disp.protocol!://%web_farm_name%/bi/{R:0}"" /commit 1>nul
:: #######

:skip_wf

echo ........Event Studio
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /-"[@4,name='Event Studio']" 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /+"[@4,name='Event Studio']" /commit 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Event Studio'].match.url:^(ags|cr1|prompting|ccl|common|skins|ps)/(.*)" /commit 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Event Studio'].action.type:"Rewrite"" /commit 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Event Studio'].action.url:"/%alias%/{R:0}"" /commit 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Event Studio'].stopProcessing:"true"" /commit 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /+"[name='Event Studio'].conditions.[input='{HTTP_REFERER}',pattern='v1/disp']" /commit 1>nul

echo ........Report Viewer
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /-"[@5,name='Report Viewer']" 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules /+"[@5,name='Report Viewer']" /commit 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Report Viewer'].match.url:^rv/(.*)" /commit 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Report Viewer'].action.type:"Rewrite"" /commit 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Report Viewer'].action.url:"/%alias%/{R:0}"" /commit 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Report Viewer'].stopProcessing:"true"" /commit 1>nul

:: SSO 
:: If enable_sso=False then SSO login and Legacy SSO will be disabled
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='SSO Login'].enabled:"%enable_SSO%"" 1>nul
%appcmd_path% set config "%web_site%/%alias%/bi" -section:system.webServer/rewrite/rules "/[name='Legacy SSO'].enabled:"%enable_SSO%"" 1>nul
echo. 
echo IIS configuration script has completed.

:eof
timeout 4 1>nul